@@ -7,6 +7,7 @@ import voluptuous as vol
 from ledfx.config import _default_wled_settings
 from ledfx.effects.audio import AudioInputSource
 from ledfx.utils import AVAILABLE_FPS, generate_title
+from ledfx.virtuals import Virtuals
 
 TYPES_MAP = {
     int: "integer",
@@ -52,6 +53,9 @@ PERMITTED_KEYS = {
         "global_transitions",
         "global_brightness",
         "melbank_collection",
+        "flush_on_deactivate",
+        "ui_brightness_boost",
+        "startup_scene_id",
     ),
 }
 
@@ -142,6 +146,16 @@ def convertToJsonSchema(schema):
     ):
         return {"type": "color", "gradient": True}
 
+    elif (
+        callable(schema)
+        and getattr(schema, "__name__", None) == "virtual_id_validator"
+    ):
+        return {
+            "type": "string",
+            "enum": Virtuals.get_virtual_ids(),
+            "names": Virtuals.get_virtual_names(),
+        }
+
     elif isinstance(schema, vol.All):
         val = {}
         for validator in schema.validators:
