@@ -6,7 +6,7 @@ from aiohttp import web
 from ledfx.api import RestEndpoint
 from ledfx.color import parse_color, validate_color
 from ledfx.config import save_config
-from ledfx.virtuals import update_effect_config
+from ledfx.effects.oneshots.oneshot import Flash
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -30,6 +30,55 @@ class VirtualsToolsEndpoint(RestEndpoint):
         """
         return await self.request_success("info", f"Available tools: {TOOLS}")
 
+    async def post(self, virtual_id, request) -> web.Response:
+        """
+        Uses the specified virtual tool on the virtual.
+
+        Args:
+            virtual_id (str): The ID of the virtual.
+            request (web.Request): The request object containing the `tool` to use.
+
+        Returns:
+            web.Response: The HTTP response object.
+        """
+        virtual = self._ledfx.virtuals.get(virtual_id)
+        if virtual is None:
+            return await self.invalid_request(
+                f"Virtual with ID {virtual_id} not found"
+            )
+
+        try:
+            data = await request.json()
+        except JSONDecodeError:
+            return await self.json_decode_error()
+
+        tool = data.get("tool")
+
+        if tool is None:
+            return await self.invalid_request(
+                'Required attribute "tool" was not provided'
+            )
+
+        if tool not in TOOLS:
+            return await self.invalid_request(f"Tool {tool} is not in {TOOLS}")
+
+        if tool == "oneshot":
+            color = parse_color(validate_color(data.get("color", "white")))
+            ramp = data.get("ramp", 0)
+            hold = data.get("hold", 0)
+            fade = data.get("fade", 0)
+            brightness = min(1, max(0, data.get("brightness", 1)))
+
+            result = virtual.add_oneshot(
+                Flash(color, ramp, hold, fade, brightness)
+            )
+
+            if result is False:
+                return await self.invalid_request("oneshot failed")
+
+        response = {"status": "success", "tool": tool}
+        return await self.bare_request_success(response)
+
     async def put(self, virtual_id, request) -> web.Response:
         """
         Uses the specified virtual tool on the virtual.
@@ -102,26 +151,14 @@ class VirtualsToolsEndpoint(RestEndpoint):
                 )
 
         if tool == "oneshot":
-            color = data.get("color")
-            if color is None:
-                return await self.invalid_request(
-                    "Required attribute for oneshot, color was not provided"
-                )
+            result = False
+            for oneshot in virtual.oneshots:
+                if isinstance(oneshot, Flash):
+                    oneshot.active = False
+                    result = True  # return True if there was at least one oneshot Flash to disable
 
-            ramp = data.get("ramp", 0)
-            hold = data.get("hold", 0)
-            fade = data.get("fade", 0)
-
-            if sum(ramp, hold, fade) == 0:
-                return await self.invalid_request(
-                    "At least one of ramp, hold or fade must be greater than 0"
-                )
-
-            result = virtual.oneshot(
-                parse_color(validate_color(color)), ramp, hold, fade
-            )
             if result is False:
-                return await self.invalid_request("oneshot failed")
+                return await self.invalid_request("oneshot was not found")
 
         if tool == "copy":
             # copy the config of the specified virtual instance to all virtuals listed in the target payload
@@ -158,9 +195,7 @@ class VirtualsToolsEndpoint(RestEndpoint):
                 except (ValueError, RuntimeError) as msg:
                     continue
 
-                update_effect_config(
-                    self._ledfx.config, dest_virtual_id, effect
-                )
+                dest_virtual.update_effect_config(effect)
                 updated += 1
 
             if updated > 0:
