@@ -1,6 +1,5 @@
 import logging
 import os
-import urllib.request
 
 import voluptuous as vol
 from PIL import Image
@@ -8,7 +7,7 @@ from PIL import Image
 from ledfx.consts import LEDFX_ASSETS_PATH
 from ledfx.effects.audio import AudioReactiveEffect
 from ledfx.effects.twod import Twod
-from ledfx.utils import get_icon_path
+from ledfx.utils import get_icon_path, open_gif
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -37,7 +36,7 @@ class Imagespin(Twod):
                 default=0.5,
             ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=1.0)),
             vol.Optional(
-                "Min Size",
+                "min_size",
                 description="The minimum size multiplier for the image",
                 default=0.3,
             ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=1.0)),
@@ -57,7 +56,7 @@ class Imagespin(Twod):
                 default=False,
             ): bool,
             vol.Optional(
-                "url source", description="Load image from", default=""
+                "image_source", description="Load image from", default=""
             ): str,
         }
     )
@@ -70,7 +69,7 @@ class Imagespin(Twod):
         super().config_updated(config)
 
         self.clip = self._config["clip"]
-        self.min_size = self._config["Min Size"]
+        self.min_size = self._config["min_size"]
         self.power_func = self.POWER_FUNCS_MAPPING[
             self._config["frequency_range"]
         ]
@@ -86,28 +85,15 @@ class Imagespin(Twod):
             getattr(data, self.power_func)() * self._config["multiplier"] * 2
         )
 
-    def open_image(self, image_path):
-        try:
-            if image_path.startswith("http://") or image_path.startswith(
-                "https://" or image_path.startswith("file://")
-            ):
-                with urllib.request.urlopen(image_path) as url:
-                    return Image.open(url)
-            else:
-                return Image.open(image_path)  # Directly open for local files
-        except Exception as e:
-            _LOGGER.error("Failed to open image: %s", e)
-            return None
-
     def do_once(self):
         super().do_once()
         if self._config["pattern"]:
             url_path = f"{os.path.join(LEDFX_ASSETS_PATH, 'test_images', 'TVTestPattern.png')}"
         else:
-            url_path = self._config["url source"]
+            url_path = self._config["image_source"]
 
         if url_path != "":
-            self.bass_image = self.open_image(url_path)
+            self.bass_image = open_gif(url_path)
             if self.bass_image:
                 _LOGGER.info(f"pre scaled {self.bass_image.size}")
 
