@@ -50,7 +50,7 @@ class Texter2d(Twod, GradientEffect):
     CATEGORY = "Matrix"
     # add keys you want hidden or in advanced here
     HIDDEN_KEYS = Twod.HIDDEN_KEYS + []
-    ADVANCED_KEYS = Twod.ADVANCED_KEYS + []
+    ADVANCED_KEYS = Twod.ADVANCED_KEYS + ["resize_method", "deep_diag"]
 
     CONFIG_SCHEMA = vol.Schema(
         {
@@ -123,14 +123,14 @@ class Texter2d(Twod, GradientEffect):
             ): vol.All(vol.Coerce(float), vol.Range(min=0.01, max=0.3)),
             vol.Optional(
                 "multiplier",
-                description="general multiplier slider for development",
+                description="multiplier of audio effect injection",
                 default=1,
             ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=10)),
             vol.Optional(
                 "speed_option_1",
                 description="general speed slider for text effects",
                 default=1,
-            ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=2)),
+            ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=3)),
         },
     )
 
@@ -260,9 +260,21 @@ class Texter2d(Twod, GradientEffect):
         # each word will be offset by the word before and a space
         # first we need the first word width in screen units
 
-        offset = 1
         self.base_speed = self.speed_option_1
 
+        if self.base_speed != 0:
+            offset = 1
+        else:
+            # we want to center the text, so we need to know the overall sentence size
+            # we can then calculate the offset to center the text
+            sentence_width = 0
+            for idx, word in enumerate(self.sentence.wordblocks):
+                sentence_width += word.w_width
+                sentence_width += self.sentence.space_block.w_width
+            # remove the trailing space implication
+            sentence_width -= self.sentence.space_block.w_width
+            offset = -sentence_width / 2
+
         for idx, word in enumerate(self.sentence.wordblocks):
             offset += word.w_width / 2
             word.pose.set_vectors(offset, 0, 0, 1, 10000)
@@ -277,8 +289,10 @@ class Texter2d(Twod, GradientEffect):
             < -1
         ):
             self.side_scroll_init()
+            # call the set_fallback function of the parent virtual as we completed a cycle
+            self._virtual.fallback_fire_set()
         for word in self.sentence.wordblocks:
-            if self.option_1:
+            if self.option_1 and self.base_speed != 0:
                 word.pose.d_pos = (
                     self.base_speed
                     + (self.lows_impulse * self.multiplier * self.base_speed),
