@@ -6,6 +6,7 @@ import importlib
 import inspect
 import ipaddress
 import logging
+import math
 import os
 import pkgutil
 import re
@@ -33,6 +34,7 @@ from platform import (
 from subprocess import PIPE, Popen
 from typing import Callable
 
+import netifaces
 import numpy as np
 import PIL.Image as Image
 import PIL.ImageFont as ImageFont
@@ -40,6 +42,7 @@ import requests
 import voluptuous as vol
 from dotenv import load_dotenv
 
+from ledfx.color import LEDFX_GRADIENTS
 from ledfx.config import save_config
 from ledfx.consts import LEDFX_ASSETS_PATH, PROJECT_VERSION
 
@@ -221,6 +224,34 @@ def get_local_ip():
         sock.close()
 
 
+def check_if_ip_is_broadcast(thisip):
+    """
+    Check if provided IP is the broadcast address of
+    one of the network interfaces
+
+    Returns:
+        True if IP is a broadcast address, False otherwise.
+
+    """
+    try:
+        # iterate over all interfaces
+        for iface in netifaces.interfaces():
+            iface = netifaces.ifaddresses(iface)
+            # iterate over all ipv4 address (if available) for this interface
+            if netifaces.AF_INET in iface:
+                for ip in iface[netifaces.AF_INET]:
+                    # check if a broadcast address is set and compare
+                    if "broadcast" in ip and ip["broadcast"] == thisip:
+                        return True
+
+        # no matching broadcast address found
+        return False
+
+    except OSError as e:
+        _LOGGER.warning(f"Unable to check if ip is a broadcast address: {e}")
+        return False
+
+
 def async_fire_and_return(coro, callback, timeout=10):
     """
     Run some async code in the core event loop with a callback to handle the result
@@ -340,8 +371,9 @@ class WLED:
         """
             Uses a JSON API call to determine if the device is WLED or WLED compatible
             and return its config.
-            Specifically searches for "WLED" in the brand json - currently all major
-            branches/forks of WLED contain WLED in the branch data.
+            As of WLED 0.15 brand is now being used to indicate specific variants of WLED
+            The brand field can no longer be depended on to indicate a compatible device
+            We now only check that brand exists as a field in the config
         Returns:
             config: dict, with all wled configuration info
         """
@@ -354,11 +386,14 @@ class WLED:
 
         wled_config = response.json()
 
-        if not wled_config["brand"] in "WLED":
-            msg = f"WLED {self.ip_address}: Not a compatible WLED brand '{wled_config['brand']}'"
-            raise ValueError(msg)
+        if "brand" not in wled_config:
+            raise ValueError(
+                f"WLED {self.ip_address}: Device is not WLED compatible"
+            )
 
-        _LOGGER.info(f"WLED {self.ip_address}: Received config")
+        _LOGGER.info(
+            f"WLED compatible device brand:{wled_config['brand']} at {self.ip_address} configuration received"
+        )
 
         return wled_config
 
@@ -707,7 +742,10 @@ def generate_id(name):
             str: The converted ID.
     """
     part1 = re.sub("[^a-zA-Z0-9]", " ", name).lower()
-    return re.sub(" +", " ", part1).strip().replace(" ", "-")
+    result = re.sub(" +", " ", part1).strip().replace(" ", "-")
+    if result == "":
+        result = "default"
+    return result
 
 
 def generate_title(id):
@@ -1043,7 +1081,10 @@ class RegistryLoader:
         found = self.discover_modules(package)
         _LOGGER.debug(f"Importing {found} from {package}")
         for name in found:
-            importlib.import_module(name)
+            try:
+                importlib.import_module(name)
+            except ModuleNotFoundError as e:
+                _LOGGER.warning(f"Failed to import {name} from {package}: {e}")
 
     def discover_modules(self, package):
         """Discovers all modules in the package"""
@@ -1409,6 +1450,13 @@ def extract_positive_integers(s):
     return list({int(num) for num in numbers if int(num) >= 0})
 
 
+def extract_uint8_seq(s):
+    # Use regular expression to find all sequences of digits
+    numbers = re.findall(r"\d+", s)
+    # discard anything outside of uint8 range
+    return [int(num) for num in numbers if 0 <= int(num) <= 255]
+
+
 def clip_at_limit(numbers, limit):
     # Keep only values that are less than the limit
     return [num for num in numbers if num < limit]
@@ -1420,7 +1468,8 @@ def open_gif(gif_path):
 
     Args:
         gif_path: str
-            path to gif file or url
+            path to gif, webp, png or jpg file or url
+            Can handle any image source that PIL is capable of, not just gif
     Returns:
         Image: PIL Image object or None if failed to open
     """
@@ -1429,13 +1478,17 @@ def open_gif(gif_path):
         if gif_path.startswith("http://") or gif_path.startswith("https://"):
             with urllib.request.urlopen(gif_path) as url:
                 gif = Image.open(url)
-                _LOGGER.debug("Remote GIF downloaded and opened.")
-                return gif
-
+                _LOGGER.debug("Remote image source downloaded and opened.")
         else:
             gif = Image.open(gif_path)  # Directly open for local files
-            _LOGGER.debug("Local GIF opened.")
-            return gif
+            _LOGGER.debug("Local image source opened.")
+
+        # protect against single frame image like png, jpg
+        if not hasattr(gif, "n_frames"):
+            gif.n_frames = 1
+
+        return gif
+
     except Exception as e:
         _LOGGER.warning(f"Failed to open gif : {gif_path} : {e}")
         return None
@@ -1518,7 +1571,18 @@ def get_font(font_list, size):
 
 
 def generate_default_config(ledfx_effects, effect_id):
-    return ledfx_effects.get_class(effect_id).get_combined_default_schema()
+    """
+    Generate config out of the schema for an effect to use as a defualt
+
+    Any manipulations must be made in here, such as expanding gradient strings to fully defined gradients
+    """
+    config = ledfx_effects.get_class(effect_id).get_combined_default_schema()
+    gradient = config.get("gradient", None)
+    gradient_str = LEDFX_GRADIENTS.get(gradient, None)
+    if gradient_str:
+        config["gradient"] = gradient_str
+        config["gradient_name"] = gradient
+    return config
 
 
 def inject_missing_default_keys(presets, defaults):
@@ -1583,33 +1647,52 @@ def log_packages():
         _LOGGER.debug(f"{dist.metadata['name']} : {dist.version}")
 
 
-def is_package_installed(package_name):
+def is_package_installed(package_name: str, import_name: str = None) -> bool:
     """
-    Check if a Python package is installed.
+    Check if a package is available in the environment.
 
     Args:
-        package_name (str): The name of the package to check.
+        package_name (str): The name used for pip installation (e.g., 'python-mbedtls').
+        import_name (str): The actual importable module name (e.g., 'mbedtls').
 
     Returns:
-        bool: True if the package is installed, False otherwise.
+        bool: True if the package is importable, False otherwise.
     """
-    try:
-        metadata.distribution(package_name)
-        return True
-    except ModuleNotFoundError:
+    import_name = import_name or package_name
+
+    # Try to get import spec
+    spec = importlib.util.find_spec(import_name)
+    if spec is None:
+        _LOGGER.info(
+            f"Optional dependency '{package_name}' not found (import name: '{import_name}')."
+        )
         return False
 
+    # Try to get version info
+    try:
+        version = metadata.version(package_name)
+    except metadata.PackageNotFoundError:
+        version = "unknown"
+
+    path = spec.origin or "unknown"
+
+    _LOGGER.info(f"Optional dependency '{package_name}' is installed:")
+    _LOGGER.info(f"  ├── Version: {version}")
+    _LOGGER.info(f"  └── Path:    {path}")
+    return True
+
 
 def check_optional_dependencies():
     """
-    Check for optional dependencies and log if they are not installed.
+    Check for optional dependencies and log their availability, versions, and paths.
     """
-    dependencies = ["psutil", "python-mbedtls"]
-    for dependency in dependencies:
-        if is_package_installed(dependency):
-            _LOGGER.info(f"Optional dependency '{dependency}' installed.")
-        else:
-            _LOGGER.info(f"Optional dependency '{dependency}' not installed.")
+    OPTIONAL_DEPENDENCIES = {
+        "psutil": "psutil",
+        "python-mbedtls": "mbedtls",
+        # Add more if needed
+    }
+    for package_name, import_name in OPTIONAL_DEPENDENCIES.items():
+        is_package_installed(package_name, import_name)
 
 
 class PerformanceAnalysis:
@@ -1804,3 +1887,308 @@ class UpdateChecker:
         return UpdateChecker._get_attribute_if_update_succeeded(
             PROJECT_VERSION != UpdateChecker.get_latest_version()
         )
+
+
+# pre stuff this to prevent per frame
+log_256 = np.log(256)
+
+
+def pixels_boost(pixels, compression_factor, max_floor):
+    # Compute the new floor based on the compression factor
+    floor = max_floor * compression_factor
+
+    # Precompute constants and reduce repeated computations
+    factor = 255 - floor
+    blend_factor = 1 - compression_factor
+
+    # Logarithmic compression: map pixel_array from [0, 255] to [floor, 255]
+    log_compressed = floor + (np.log1p(pixels) / log_256) * factor
+
+    # Apply the blend only where pixels are greater than 1, else leave unchanged
+    boosted = np.where(
+        pixels > 1,
+        blend_factor * pixels + compression_factor * log_compressed,
+        pixels,
+    )
+
+    return boosted
+
+
+def dump_pixels(pixels, shape):
+    """
+    Dump a RGB pixels np array into a PIL image and launch a window to display it.
+    This is for adhoc debug only
+
+    Parameters:
+    - pixels: 1D array of concatenated RGB pixel values
+    - shape: tuple, the desired shape of the 2D image (height, width)
+
+    Returns:
+    - none
+    """
+
+    # Reshape the 1D pixel array into (height, width, 3) for RGB
+    reshaped_pixels = pixels.reshape((shape[0], shape[1], 3))
+    # Convert the numpy array back into a Pillow image
+    image = Image.fromarray(reshaped_pixels.astype(np.uint8), "RGB")
+
+    image.show()
+
+
+def resize_pixels(pixels, old_shape, new_shape):
+    """
+    Resizes a 1D pixel array that represents a for 1D or 2D image by interpolating it in 2D space using PIL.
+
+    Profiled as at least as good as or better than interpolate_pixels() for performance even with the overhead of creating a PIL image.
+
+    Parameters:
+    - pixels: 1D array of concatenated RGB pixel values
+    - old_shape: tuple, the original shape of the 2D image (height, width)
+    - new_shape: tuple, the desired shape of the resized 2D image (height, width)
+
+    Returns:
+    - A resized 1D pixel array
+    """
+    # Reshape the 1D array into a 2D image (height, width, 3)
+    pixel_matrix = pixels.reshape((old_shape[0], old_shape[1], 3)).astype(
+        np.uint8
+    )
+
+    # Create a PIL image from the pixel data
+    image = Image.fromarray(pixel_matrix)
+
+    # Resize the image using PIL (bilinear interpolation is default)
+    resized_image = image.resize((new_shape[1], new_shape[0]), Image.BILINEAR)
+
+    # Convert the resized image back to a numpy array
+    resized_pixel_matrix = np.array(resized_image)
+
+    # Flatten the resized pixel matrix back to 1D
+    return resized_pixel_matrix.reshape(-1, 3)
+
+
+def shape_to_fit_len(max_len, shape, pixels_len):
+    """Converts the shape of a visualisation to obey constraints
+       The max_len pixels which is a system variable a user can change
+       Additionally a max dimension of 64 in rows or columns is enforced
+       As the shape has already been reduced to max pixels, both CANNOT
+       be greate than 64
+
+    Args:
+        max_len : The maximum number of pixels allowed in the final visualisation
+        shape : The current shape of the visualisation before resizing
+        pixels_len : The number of pixels in the current visualisation
+
+    Returns:
+        new_shape : The shape calculated to use in resizing
+        pixels_len : The number of pixels implied in the new_shape
+    """
+
+    if shape[0] > 1:
+        # this is a 2d visualisation
+        pixels_len = shape[0] * shape[1]
+        reduction_ratio = math.sqrt(pixels_len / max_len)
+        new_rows = shape[0] / reduction_ratio
+        new_cols = shape[1] / reduction_ratio
+
+        # protect against extreme shapes
+        # see function description for magic number 64
+        if new_rows > 64:
+            reduction_ratio = 64 / new_rows
+            new_rows = 64
+            new_cols = new_cols * reduction_ratio
+        elif new_cols > 64:
+            reduction_ratio = 64 / new_cols
+            new_cols = 64
+            new_rows = new_rows * reduction_ratio
+
+        # protect from less than 1 values
+        if new_rows < 1.0:
+            new_shape = (1, new_cols)
+        elif new_cols < 1.0:
+            new_shape = (new_rows, 1)
+        else:
+            new_shape = (
+                int(new_rows),
+                int(new_cols),
+            )
+    else:
+        # this is a 1d visualisation
+        # pixels_len will be unchanged
+        new_shape = (1, max_len)
+
+    return new_shape, pixels_len
+
+
+class Teleplot:
+    """
+    Helper class for the use of vscode Teleplot extension
+
+    import and inline call
+    Teleplot.send("variable_name:value")
+    """
+
+    # only one socket for all instances
+    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+
+    @staticmethod
+    def send(string):
+        """Send string to teleplot
+
+        https://github.com/nesnes/teleplot
+
+        Args:
+            string: The string carrying the teleplot data of the format as per link above
+        """
+
+        try:
+            Teleplot.sock.sendto(string.encode(), ("127.0.0.1", 47269))
+        except Exception as e:
+            _LOGGER.error(f"Failed to send data to teleplot: {e}")
+
+
+def aggressive_top_end_bias(x, boost):
+    """
+    Apply an aggressive top-end bias to input values.
+
+    This function blends between a linear curve and a non-linear curve that
+    pulls higher values closer to 1.0 as boost increases.
+
+    - At boost = 0: the output is equal to the input (linear).
+    - At boost = 1: higher input values are strongly biased toward 1.0.
+
+    The transformation is applied element-wise if `x` is a NumPy array.
+
+    Parameters:
+        x (float or np.ndarray): Input value(s) between 0 and 1.
+        boost (float): Boost factor between 0 and 1.
+
+    Returns:
+        float or np.ndarray: Transformed value(s), same shape and type as `x`.
+    """
+
+    aggressive_curve = 1 - (1 - x) ** 4  # Adjust power for curve steepness
+    return (1 - boost) * x + boost * aggressive_curve
+
+
+def get_sorted_physical_ips() -> list[str]:
+    """
+    Returns a sorted list of local non-loopback IPv4 addresses from physical interfaces.
+    Sorts by:
+        1. Primary outbound IP (used for external connections)
+        2. Interface activity (bytes sent + received)
+    Logs interface decisions and IPs for triage.
+    """
+    try:
+        import psutil
+
+        ip_usage_list = []
+
+        # Heuristics for physical interfaces
+        physical_keywords = [
+            "eth",
+            "en",
+            "ens",
+            "eno",
+            "enp",
+            "wlan",
+            "wl",  # Linux
+            "Wi-Fi",
+            "Ethernet",
+            "Local Area",  # Windows
+            "en",
+            "bridge",  # macOS
+        ]
+
+        _LOGGER.info("Starting local IP discovery")
+
+        stats = psutil.net_if_stats()
+        counters = psutil.net_io_counters(pernic=True)
+
+        for iface_name, iface_addrs in psutil.net_if_addrs().items():
+            if not any(keyword in iface_name for keyword in physical_keywords):
+                _LOGGER.info(f"Skipping non-physical interface: {iface_name}")
+                continue
+            if iface_name not in stats or not stats[iface_name].isup:
+                _LOGGER.info(f"Skipping inactive interface: {iface_name}")
+                continue
+
+            _LOGGER.info(f"Inspecting interface: {iface_name}")
+            for addr in iface_addrs:
+                if addr.family == socket.AF_INET:
+                    if addr.address.startswith("127."):
+                        _LOGGER.info(
+                            f"Skipping loopback address on {iface_name}: {addr.address}"
+                        )
+                        continue
+                    counter = counters.get(iface_name)
+                    usage = (
+                        (counter.bytes_sent + counter.bytes_recv)
+                        if counter
+                        else 0
+                    )
+                    _LOGGER.info(
+                        f"Discovered IP {addr.address} on {iface_name} with usage {usage}"
+                    )
+                    ip_usage_list.append((usage, addr.address))
+    except Exception as e:
+        _LOGGER.warning(f"Failed to get network interface info: {e}")
+        primary_ip = get_primary_ip()
+        if primary_ip:
+            return [primary_ip]
+        else:
+            _LOGGER.warning(
+                "No network interfaces found and primary IP detection failed."
+            )
+            return []
+
+    ip_usage_list.sort(reverse=True)
+    sorted_ips = [ip for _, ip in ip_usage_list]
+
+    # Try to determine the primary IP based on routing
+    primary_ip = get_primary_ip()
+
+    if primary_ip is not None:
+        if primary_ip in sorted_ips:
+            sorted_ips.remove(primary_ip)
+        sorted_ips.insert(0, primary_ip)
+
+    _LOGGER.info(f"Final sorted IP list: {sorted_ips}")
+    return sorted_ips
+
+
+def get_primary_ip() -> str:
+    """
+    Returns the primary local IPv4 address used for outbound traffic.
+    This works across Windows, macOS, Linux, and Android (Termux, etc.).
+    """
+    try:
+        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        s.settimeout(1.0)
+        s.connect(
+            ("8.8.8.8", 80)
+        )  # Doesn't send packets; just gets routing info
+        ip = s.getsockname()[0]
+        s.close()
+        _LOGGER.info(f"Primary outbound IP detected: {ip}")
+        return ip
+    except Exception as e:
+        _LOGGER.warning(f"Primary IP detection via socket failed: {e}")
+        return None  # no fallback
+
+
+def nonlinear_log(x, power=3):
+    """
+    Apply a nonlinear logarithmic transformation to the input value(s).
+    This function applies a transformation that is similar to a logarithm but
+    allows for a customizable power to control the steepness of the curve.
+    - At power = 1: the output is equal to the input (linear).
+    - At power > 1: higher input values are pulled closer to 1.0.
+    The transformation is applied element-wise if `x` is a NumPy array.
+    Parameters:
+        x (float or np.ndarray): Input value(s) between 0 and 1.
+        power (float): Power factor to control the steepness of the curve.
+    Returns:
+        float or np.ndarray: Transformed value(s), same shape and type as `x`.
+    """
+    return np.sign(x) * (abs(x) ** power)
